//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Joon Suk Lee (jl), Kee-Eung Kim (kek) 
//  FILE: PCState.H
//  PROJ: COKO
//  DATE: 8/27/96                                                              
//
// SYNOPSIS
//  Class definitions for CState.
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef _PCSTATE_H_
#define _PCSTATE_H_

#include "PCIEnvStack.H"

class CPattern;
class DEPList;
class CIdent;
class CIEnvValueType;
class CIEnvStack;
class CIEnvStackDataType;

//////////////////////////////////////////////////////////////////////////////
//
//  CLASS: CState
//
// SYNOPSIS
//  CState is a class for abstraction of the state of program. All the 
//  statement executions depend passing and receiving CState.
//
//  This class actually wraps four tuple structure:
//   * _Store : Local root of the query tree
//   * _IEnv : Stack of identifier search stack
//   * _CurrPattern : Pattern specified in the latest match (or match
//        each child) statement.
//   * _Success : Return flag for specifiying whether the statement execution
//        succeeded or not.
//
//////////////////////////////////////////////////////////////////////////////

class CState {

protected:
  
  NonType *_Store;           
  // Local root pointer

  NonType *_rootStore;           
  // Global root pointer
  
  CIEnvStack * _IEnv;
  // Identifier search stack
  
  DEPList * _InvDEPList;
  // Inverse dependency list for current
  // current only needs inverse dependancy list
  // dependancy list for current is meaningless

  CPattern *_CurrPattern;
  // Pattern in the last match (each child) stmt
  
  int _Success;
  // Return flag

public:

  CState(NonType *, CIEnvStack *, int);
  // Constructor

  ~CState();                            
  // Destructor
  
  NonType *Store(void);
  // Returns _Store
  
  NonType *rootStore(void);
  // Returns _rootStore

  CIEnvStack *IEnv(void);
  // Returns _IEnv

  void SetIEnv(CIEnvStack*);
  // set _IEnv

  int ISuccess(void);
  // Returns _Success
  
  void SetStore(NonType *);
  // Modifier for _Store
  
  void SetRootStore(NonType *);
  // Modifier for _rootStore

  void SetSuccess(int);
  // Modifier for _Success
  
  CIdent * IFind(const char *);
  // Search only for the top frame (another stack type) of the stack

  int IInsert(const char*, CIdent*);
  // Identifier inserter (inserts into
  // the top frame of the stack.
 
  void INew(void);
  // Pushes an empty frame (another stack type) into the stack

  void IOld(void);
  // Pop up the top element of the stack

  void IModify(char *, NonType *, int);
  // Modifier for a particular
  // identifier


  void IDeleteDeep(char *);                  
  // Delete identifier from the stack
  // Search for the entire stack.

  CIdent * IFindDeep(char *);
  // Search for the entire stack.

  CIdent * IFindDeepDB(char *);
  // Search for the entire stack for given key.
  // ADDED BY DJA WINTER 2002

  CIdent * IFindFromHere(char *, CIEnvStackDataType*);
  // Search for entire stack starting from the indicated point to down.

  void IModifyFromHere(char*, NonType*, int, CIEnvStackDataType*);
  // same as IModify but this one search the entire stack  
  // starting from the indicated point to down.

  CIEnvValueType * IFirst(void);
  // Returns the root node value of 
  // identifier search tree.
  
  void SetCurrentPattern(CPattern *);   
  // Modifier for _CurrPattern
  
  CPattern * CurrentPattern(void);
  // Returns _CurrPattern
  
  int IInsertDep(const char* key, char* a );
  // find a node with key then insert a to its dependency list

  int IInsertInvDep(const char*, char*);
  // find a node with key then insert a to its inverse dependency list

  void IDeleteInvDep(const char*);
  // Find a node with key then retrieve all the key values 
  // in its inverse dependency list. With all the retrieved key values, 
  // search and delete all the entries from the stack.

  int IEmptyInvDep(const char*);
  // First search the stack with a given key.
  // Is a inverse dependancy list stored in the found node empty?

  void SetPatternDeep(const char * key, CPattern * p);
  // First search the stack with a given key.
  // Modify a pattern stored in the found node.

  CPattern* GetPatternFromHere(const char*, CIEnvStackDataType*);
  // First search the stack with a given key.
  // Start searching from the startingPt.
  // Searching from the starting point instead of the top of the stack 
  // makes code more efficient.
  // Return a pattern stored in the found node


  // methods for _InvDEPList.

  void CInsertInvDep(char*);
  // Insert a char* entry into the _InvDEPList

  void CDeleteInvDep();
  // Retrieve all the key values from _InvDEPList.
  // With all the retrieved key values, search and delete 
  // all the entries from the stack.

  int CEmptyInvDep();
  // is _InvDEPList empty?

  void CCleanInvDep();
  // Clean _InvDEPList. Make _InvDEPList as it was first created.
  // To do so, just delete old _InvDEPList and assign new DEPList to
  // _InvDEPList.

  DEPList* GetInvDep();
  // Returns _InvDEPList

  void SetInvDep(DEPList* a);
  // Modify _InvDEPList
  
  void DelInvDep();
  // delete _InvDEPList

};

#endif
